/***************************************************************************
 *   Copyright (C) 2004 by Joris Guisson                                   *
 *   joris.guisson@pandora.be                                       *
 *                                                                         *
 ***************************************************************************/
#include <klocale.h>

#include <qlayout.h>
#include <qlabel.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <kiconloader.h>
#include <kglobal.h>
#include "musicmanprefdlg.h"
#include "mmsettings.h"
#include "kmusicmanager.h"
#include "mainwindow.h"
#include "player/musicplayer.h"
#include "player/playerfactoryrepository.h"
#include "view/musiclistview.h"


MusicManPrefDlg::MusicManPrefDlg(KMusicManager* mm)
		: KDialogBase(IconList, i18n("Preferences"),Ok|Apply|Cancel, Ok),
		mm(mm)
{
	// this is the base class for your preferences dialog.  it is now
	// a Treelist dialog.. but there are a number of other
	// possibilities (including Tab, Swallow, and just Plain)
	this->enableButtonSeparator(true);
	setInitialSize(QSize(500,400),true);
	KIconLoader* il = KGlobal::iconLoader();
	m_page_one_icon = il->loadIcon("kmusicmanager",KIcon::Desktop);
	QFrame *frame;
	frame = addPage(i18n("General"), i18n("General"),m_page_one_icon);
	m_page_one = new MusicManPrefPageOne(mm,frame);
	m_page_one->setMinimumSize(400,300);

	//frame = addPage(i18n("Second Page"), i18n("Page Two Options"));
	//m_pageTwo = new MusicManPrefPageTwo(frame);
}


MusicManPrefDlg::~MusicManPrefDlg()
{}

void MusicManPrefDlg::slotOk()
{
	m_page_one->apply(mm);
	accept();
}

void MusicManPrefDlg::slotApply()
{
	m_page_one->apply(mm);
}


//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////

MusicManPrefPageOne::MusicManPrefPageOne(KMusicManager* mm,QWidget *parent)
		: QFrame(parent),mms(0)
{
	QVBoxLayout *layout = new QVBoxLayout(this);
	layout->setAutoAdd(true);

	mms = new MMSettings(this);
	mms->setViewConf(MusicListView::getViewConfig());
	mms->systray_ch->setChecked(mm->getMainWindow()->isSystemTrayIconEnabled());
	QComboBox* cb = mms->mm_backend;
	
	PlayerFactoryRepository* pfr = mm->getPlayer()->getFactoryRepository();
	int curr = 0;
	int j = 0;
	for (PlayerFactoryRepository::const_iterator i = pfr->begin();i != pfr->end();i++)
	{
		const PlayerFactory* fac = i->second;
		if (fac->getName() == mm->getPlayer()->getBackendName())
			curr = j;
		cb->insertItem(fac->getDescriptiveName());
		j++;
	}
	
	cb->setCurrentItem(curr);
}

void MusicManPrefPageOne::apply(KMusicManager* mm)
{
	mm->getMainWindow()->enableSysTrayIcon(mms->systray_ch->isChecked());
	MusicListView::setViewConfig(mms->vconf);
	mm->getMainWindow()->updateChildViews();
	
	PlayerFactoryRepository* pfr = mm->getPlayer()->getFactoryRepository();
	int curr = mms->mm_backend->currentItem();
	QString b = pfr->getFactoryName(curr);
	if (b != QString::null)
	{
		mm->getPlayer()->changeBackend(b);
	}
}

#include "musicmanprefdlg.moc"
