/***************************************************************************
 *   Copyright (C) 2004 by Joris Guisson                                   *
 *   joris.guisson@student.luc.ac.be                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <qdir.h>
#include <qstringlist.h>
#include <kdirlister.h>
#include <qapplication.h>
#include "dbdir.h"
#include "musicdatabase.h"
#include "song.h"
#include "globals.h"



DBDir::DBDir(MusicDatabase* db,DBDir* parent,const QString & path,const QString & name)
	: QObject(parent),m_parent(parent),m_path(path),m_name(name),m_db(db)
{
	if (!m_path.endsWith("/"))
		m_path += "/";

	m_lister = new KDirLister();
	m_lister->setAutoUpdate(true);
//	m_lister->setNameFilter("*.mp3 *.MP3 *.ogg *.OGG *.flac *.FLAC");
	connect(m_lister,SIGNAL(completed()),this,SLOT(onListerCompleted()));
	connect(m_lister,SIGNAL(newItems(const KFileItemList& )),
			this,SLOT(onNewItems(const KFileItemList& )));
	connect(m_lister,SIGNAL(deleteItem(KFileItem* )),
			this,SLOT(onDeleteItem(KFileItem* )));
	connect(m_lister,SIGNAL(refreshItems(const KFileItemList& )),
			this,SLOT(onRefreshItems(const KFileItemList& )));

	m_lister->openURL(KURL(m_path + m_name));
}


DBDir::~DBDir()
{
	delete m_lister;
}

void DBDir::rescan()
{
	m_lister->updateDirectory(KURL(m_path + m_name));
}

void DBDir::addSong(KFileItem* fi)
{
	QString path = m_path + m_name + "/";

	QString fn = fi->name(false);
	Song* s = m_db->getSong(path + fn);
	if (!s)
	{
		s = new Song(path + fn);
		m_db->addSong(s);
		
		OutputLog() << "Adding new song " << (path + fn) << endl;
	}
	else if (s->isModifiedOnDisk())
	{
		OutputLog() << "Refereshing song " <<  (path + fn) << endl;
		s->loadInfoFromFile();
	}
}

void DBDir::addDir(KFileItem* fi)
{
	QString path = m_path + m_name + "/";

	QString name = fi->name(false);
	QString dpath = path + name;
	if (name != ".." && name != "." && !m_children.contains(dpath))
	{
		OutputLog() << "Adding dir " << dpath << endl;
		DBDir* d = new DBDir(m_db,this,path,name);
		m_children[dpath] = d;
	}
	qApp->processEvents();
}

void DBDir::onListerCompleted()
{
	QString path = m_path + m_name + "/";

	OutputLog() << "DBDir::onListerCompleted" << endl;
	m_lister->setNameFilter("*.mp3 *.MP3 *.ogg *.OGG *.flac *.FLAC");

	KFileItemList fi = m_lister->items();
	KFileItem* i = fi.first();

	while (i)
	{
		if (!i->isDir())
			addSong(i);
		else
			addDir(i);

		i = fi.next();
	}
	m_lister->setNameFilter(QString::null);
}

void DBDir::onNewItems(const KFileItemList &items)
{
	QString path = m_path + m_name + "/";

	KFileItemList & fi = const_cast<KFileItemList & > (items);
	OutputLog() << "New Items detected : " << endl;
	m_lister->setNameFilter("*.mp3 *.MP3 *.ogg *.OGG *.flac *.FLAC");
	KFileItem* i = fi.first();
	while (i)
	{
		OutputLog() << "\t" << path << i->name(false) << endl;
		if (i->isDir())
			addDir(i);
		else if (m_lister->matchesFilter(i->name(false)))
			addSong(i);

		i = fi.next();
	}
	m_lister->setNameFilter(QString::null);
}

void DBDir::onDeleteItem(KFileItem* fileItem)
{
	QString path = m_path + m_name + "/" + fileItem->name(false);
	OutputLog() << "Deleting item " << path;
	if (fileItem->isFile())
	{
		OutputLog() << " (Song)" << endl;
		m_db->removeSong(path);
	}
	else if (fileItem->isDir() && m_children.contains(path))
	{
		OutputLog() << " (Directory)" << endl;
		DBDir* sd = m_children[path];
		sd->clear();
		delete sd;
		m_children.erase(path);
	}
	else if (getFullPath() == path)
	{
		OutputLog() << " (This Directory)" << endl;
		clear();
	}
}

void DBDir::onRefreshItems(const KFileItemList &items)
{
	QString path = m_path + m_name + "/";

	KFileItemList & fi = const_cast<KFileItemList & > (items);

	m_lister->setNameFilter("*.mp3 *.MP3 *.ogg *.OGG *.flac *.FLAC");
	KFileItem* i = fi.first();
	OutputLog() << "Refreshing items : " << endl;
	while (i)
	{
		OutputLog() << "\t" << path << i->name(false) << endl;
		if (i->isDir())
			refreshDir(i);
		else if (m_lister->matchesFilter(i->name(false)))
			refreshSong(i);

		i = fi.next();
	}
	m_lister->setNameFilter(QString::null);
}

void DBDir::refreshSong(KFileItem* fi)
{
	QString path = m_path + m_name + "/" + fi->name(false);
	Song* s = m_db->getSong(path);
	if (!s)
	{
		addSong(fi);
	}
	else
	{
		s->loadInfoFromFile();
	}
	
}

void DBDir::refreshDir(KFileItem* fi)
{
	m_lister->updateDirectory(KURL(m_path + m_name + "/" + fi->name(false)));
}

void DBDir::clear()
{
	OutputLog() << "Clearing directory " << m_path << m_name << endl;
	for (QMap<QString,DBDir*>::iterator i = m_children.begin();i != m_children.end();i++)
	{
		DBDir* d = i.data();
		d->clear();
		//dirRemoved(d);
		delete d;
	}
	m_children.clear();
}

void DBDir::setDirName(const QString & name)
{
	m_name = name;
//	nameChanged(name);
}

#include "dbdir.moc"
