/***************************************************************************
 *   Copyright (C) 2004 by Joris Guisson                                   *
 *   joris.guisson@pandora.be                                       *
 *                                                                         *
 ***************************************************************************/
#include <stdio.h>
#include <errno.h>
#include <qfileinfo.h>
#include <klocale.h>
#include "musicdbexporter.h"
#include "musicdatabase.h"
#include "song.h"
#include "dbdir.h"
#include "util/error.h"

MusicDBExporter::MusicDBExporter(MusicDatabase* db) : m_db(db)
{}


MusicDBExporter::~MusicDBExporter()
{}

static void ExportString(const QString & s,FILE* fptr)
{
	unsigned int size = s.length();
	fwrite(&size,sizeof(unsigned int),1,fptr);
	fwrite(s.ascii(),sizeof(char),size,fptr);
}

static void ExportSong(Song* s,FILE* fptr)
{
	QFileInfo finfo(s->getFilename());
	SongHeader h;
	h.length = s->getLength();
	h.track = s->getAlbumNr();
	h.year = s->getYear();
	h.last_modified = finfo.lastModified().toTime_t();
	
	fwrite(&h,sizeof(SongHeader),1,fptr);
	ExportString(s->getFilename(),fptr);
	ExportString(s->getTitle(),fptr);
	ExportString(s->getArtist(),fptr);
	ExportString(s->getAlbum(),fptr);
	ExportString(s->getGenre(),fptr);
	ExportString(s->getComment(),fptr);
}


void MusicDBExporter::exportDB(const QString & file)
{
	FILE* fptr = fopen(file.ascii(),"wb");
	
	if (!fptr)
		throw Error(i18n("Error opening file %1 : %2").arg(file).arg(strerror(errno)));
	
	Header h;
	h.num_dirs = m_db->getNumDirs();
	h.num_files = m_db->getNumSongs();
	h.magic = 0x66666601;
	
	fwrite(&h,sizeof(Header),1,fptr);
	
	for (DBDirItr i = m_db->beginDirs();i != m_db->endDirs();++i)
	{
		ExportString(i.key(),fptr);
	}
	
	for (SongItr i = m_db->beginSongs();i != m_db->endSongs();++i)
	{
		ExportSong(i.data(),fptr);
	}
	
	fclose(fptr);
}
