/***************************************************************************
 *   Copyright (C) 2004 by Joris Guisson                                   *
 *   joris.guisson@pandora.be                                       *
 *                                                                         *
 ***************************************************************************/
#include <stdio.h>
#include <errno.h>
#include <list>
#include <qfileinfo.h>
#include <klocale.h>
#include "musicdbimporter.h"
#include "song.h"
#include "musicdatabase.h"
#include "util/error.h"
#include "musicdbexporter.h"
#include "globals.h"



MusicDBImporter::MusicDBImporter(MusicDatabase* db) : m_db(db)
{}


MusicDBImporter::~MusicDBImporter()
{}

static QString ImportString(FILE* fptr)
{
	QString foo;
	unsigned int size;
	fread(&size,sizeof(unsigned int),1,fptr);
	for (unsigned int i = 0;i < size;i++)
	{
		char c;
		fread(&c,sizeof(char),1,fptr);
		foo.append(c);
	}
	return foo;
}

static Song* ImportSong(FILE* fptr)
{
	SongHeader h;
	
	fread(&h,sizeof(SongHeader),1,fptr);
	QString fn = ImportString(fptr);
	
	
	Song* s = new Song(fn);
	s->setAlbumNr(h.track);
	s->setLength(h.length);
	s->setYear(h.year);
	
	s->setTitle(ImportString(fptr));
	s->setArtist(ImportString(fptr));
	s->setAlbum(ImportString(fptr));
	s->setGenre(ImportString(fptr));
	s->setComment(ImportString(fptr));
	
	QFileInfo finfo(s->getFilename());
	if (!finfo.exists())
	{
		delete s;
		return 0;
	}
	if (finfo.lastModified().toTime_t() != h.last_modified)
	{
		s->loadInfoFromFile();
	}
	else
	{
		s->setLastModified(finfo.lastModified().toTime_t());
	}
	
	return s;
}

void MusicDBImporter::importDB(const QString & filename)
{
	FILE* fptr = fopen(filename.ascii(),"rb");
	
	if (!fptr)
		throw Error(i18n("Error opening file %1 : %2").arg(filename).arg(strerror(errno)));
	
	Header h;
	fread(&h,sizeof(Header),1,fptr);
	
	if (h.magic != 0x66666601)
		throw Error(i18n("Error : database file %1 corrupt").arg(filename));
	
	std::list<QString> dirs;
	for (int i = 0;i < h.num_dirs;i++)
	{
		QString d = ImportString(fptr);
		m_db->addDir(d);
	}
	
	for (int i = 0;i < h.num_files;i++)
	{
		Song* s = ImportSong(fptr);
		if (s)
			m_db->addSong(s,false);
	}
		
	fclose(fptr);
}
