/***************************************************************************
 *   Copyright (C) 2004 by Joris Guisson                                   *
 *   joris.guisson@pandora.be                                       *
 *                                                                         *
 ***************************************************************************/
#include <qfileinfo.h>
#include <taglib/fileref.h>
#include <taglib/tag.h>
#include <taglib/tstring.h>
#include <taglib/audioproperties.h>
#include "song.h"


Song::Song(const QString & filename) : filename(filename),last_mod(0)
{
	QFileInfo fi(filename);
	dir = fi.dirPath(true);
	if (!dir.endsWith("/"))
		dir += "/";
	album_nr = 0;
	length = 0;
	year = 0;
}

Song::Song(const Song & s) : QObject(0,0)
{
	filename = s.filename;
	title = s.title;
	artist = s.artist;
	album = s.album;
	album_nr = s.album_nr;
	length = s.length;
	year = s.year;
	genre = s.genre;
	dir = s.dir;
}

Song::~Song()
{}

QString Song::getFileExtension() const
{
	int dp = filename.findRev(".");
	if (dp == -1)
		return QString::null;
	else
		return filename.mid(dp);
}

bool Song::isModifiedOnDisk() const
{
	QFileInfo finfo(filename);
	return last_mod != finfo.lastModified().toTime_t();
}

bool Song::existsOnDisk() const
{
	QFileInfo finfo(filename);
	return finfo.exists();
}

bool Song::loadInfoFromFile()
{
	TagLib::FileRef f(filename);
	if (f.isNull())
		return false;
	
	artist = f.tag()->artist().toCString();
	title = f.tag()->title().toCString();
	album = f.tag()->album().toCString();
	year = f.tag()->year();
	album_nr = f.tag()->track();
	genre = f.tag()->genre().toCString();
	comment = f.tag()->comment().toCString();
	length = f.audioProperties()->length();
	
	QFileInfo finfo(filename);
	last_mod = finfo.lastModified().toTime_t();
	emitModified();
	return true;
}

bool Song::saveInfoToFile()
{
	TagLib::FileRef f(filename);
	if (f.isNull())
		return false;
	
	f.tag()->setArtist(artist.ascii());
	f.tag()->setTitle(title.ascii());
	f.tag()->setAlbum(album.ascii());
	f.tag()->setComment(comment.ascii());
	if (year > 0)
		f.tag()->setYear(year);
	
	if (album_nr > 0)
		f.tag()->setTrack(album_nr);
	
	f.tag()->setGenre(genre.ascii());
	f.save();
	emitModified();
	return true;
}

bool Song::matchesFilter(const QString & filter)
{
	if (filter.isEmpty())
		return true;
	
	if (artist.contains(filter,true))
		return true;
	
	if (title.contains(filter,true))
		return true;
	
	if (album.contains(filter,true))
		return true;
	
	if (genre.contains(filter,true))
		return true;
	
	if (QString("%1").arg(year).contains(filter,true))
		return true;
	
	if (QString("%1").arg(album_nr).contains(filter,true))
		return true;
	
	return false;
}

void Song::emitModified()
{
	modified();
	modified(this);
}


#include "song.moc"

