/***************************************************************************
 *   Copyright (C) 2004 by Joris Guisson                                   *
 *   joris.guisson@pandora.be                                       *
 *                                                                         *
 ***************************************************************************/
#ifndef MUSICPLAYER_H
#define MUSICPLAYER_H

#include <qobject.h>
#include <qstring.h>
#include "musicmaniface.h"

class QTimer;
class Song;
class KConfig;
class PlayHistory;
class PlayQueue;
class PlayerBase;
class SongProvider;
class KToolBar;
class MusicDatabase;
class PlayerFactoryRepository;

/**
@author Joris Guisson
*/
class MusicPlayer : public QObject,virtual public MusicManIface
{
	Q_OBJECT

	PlayerBase* m_player;
	SongProvider* m_db;
	bool m_repeat,m_random; 
	QTimer* m_timer;
	float m_volume;
	PlayQueue* m_play_queue;
	PlayHistory* m_history;
	PlayerFactoryRepository* m_pfrep;
	QString m_backend_name;
	
	void playNextSong(bool at_end_of_song = false);
	void nothingPlaying();
	void addToHistory(Song*);
	Song* popHistory();
	void initVolumeControl();
public:
	MusicPlayer(MusicDatabase* mdb,QObject *parent = 0, const char *name = 0);
	virtual ~MusicPlayer();
	
	bool getRepeat() const {return m_repeat;}
	bool getRandom() const {return m_random;}
	float getVolume() const {return m_volume;}
	QString getBackendName() const {return m_backend_name;}
	PlayHistory* getHistory() {return m_history;}
	PlayQueue* getPlayQueue() {return m_play_queue;}
	PlayerFactoryRepository* getFactoryRepository() {return m_pfrep;}
	
	void saveSettings(KConfig* cfg);
	void loadSettings(KConfig* cfg);
	void setProvider(SongProvider* db);
	void createPlayToolBar(KToolBar* ptb);
	
public slots:
	void play();
	void pause();
	void stop();
	void next();
	void prev();
	void setRandom(bool on);
	void setRepeat(bool on);
	void onTimeout();
	void seek(int secs);
	void playSong(Song* s);
	void setVolume(float vol);
	void enqueueSong(Song* s);
	void changeBackend(const QString & name);

signals:
	void playing(const QString & filename);
	void elapsed(int curr_time);
	void totalSongTimeChanged(int t);
	void setPauseEnabled(bool yes);
	void setPlayEnabled(bool yes);
	void setStopEnabled(bool yes);
	void setNextEnabled(bool yes);
	void setPrevEnabled(bool yes);
	void volumeChanged(float vol);
};

#endif
