/***************************************************************************
 *   Copyright (C) 2004 by Joris Guisson                                   *
 *   joris.guisson@student.luc.ac.be                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "playerfactoryrepository.h"
#include "util/error.h"

PlayerFactoryRepository::PlayerFactoryRepository()
{}


PlayerFactoryRepository::~PlayerFactoryRepository()
{
	for (iterator i = m_pfmap.begin();i != m_pfmap.end();i++)
		delete i->second;
}

void PlayerFactoryRepository::addFactory(PlayerFactory* fac)
{
	if (!fac)
		return;
	
	iterator i = m_pfmap.find(fac->getName());
	if (i != m_pfmap.end())
		throw Error("There allready is a factory named " + fac->getName());
	
	m_pfmap[fac->getName()] = fac;
}

PlayerFactory* PlayerFactoryRepository::getFactory(const QString & name)
{
	iterator i = m_pfmap.find(name);
	return (i != m_pfmap.end()) ? i->second : 0;
}

void PlayerFactoryRepository::removeFactory(const QString & name)
{
	iterator i = m_pfmap.find(name);
	if (i != m_pfmap.end())
	{
		delete i->second;
		m_pfmap.erase(i);
	}
}

QString PlayerFactoryRepository::getFactoryName(int index) const
{
	int j = 0;
	const_iterator i = m_pfmap.begin();
	while (i != m_pfmap.end())
	{
		if (j == index)
			return i->first;
		i++; j++;
	}
	return QString::null;
}
