/***************************************************************************
 *   Copyright (C) 2004 by Joris Guisson                                   *
 *   joris.guisson@student.luc.ac.be                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef PLAYERFACTORYREPOSITORY_H
#define PLAYERFACTORYREPOSITORY_H

#include <map>
#include <qstring.h>
#include "playerfactory.h"

/**
@author Joris Guisson
@brief Keeps track of all PlayerFactory objects

This class keeps track of all PlayerFactory instances.
It uses a map to store them.
*/
class PlayerFactoryRepository
{
	std::map<QString,PlayerFactory*> m_pfmap;
public:
	/**
	 * Constructor.
	 */
	PlayerFactoryRepository();
	
	/**
	 * Destroy all PlayerFactory objects.
	 */
	virtual ~PlayerFactoryRepository();
	
	typedef std::map<QString,PlayerFactory*>::iterator iterator;
	typedef std::map<QString,PlayerFactory*>::const_iterator const_iterator;

	/**
	 * Add a factory. This will throw an Error if there
	 * allready is a factory with the same name.
	 * @param fac The PlayerFactory
	 */
	void addFactory(PlayerFactory* fac);
	
	/**
	 * Lookup a factory based on its name.
	 * @param name The name of the factory
	 * @return The PlayerFactory or 0 if no appropriate factory could be found
	 */
	PlayerFactory* getFactory(const QString & name);
	
	/**
	 * Remove a factory from the repository.
	 * @param name The name of the factory
	 */
	void removeFactory(const QString & name);
	
	/**
	 * Get a const_iterator to the beginning of the map.
	 * @return A const_iterator
	 */
	const_iterator begin() const {return m_pfmap.begin();}
	
	/**
	 * Get a const_iterator to the end of the map.
	 * @return A const_iterator
	 */
	const_iterator end() const {return m_pfmap.end();}
	
	/**
	 * Get the name of the PlayerFactory at @a index
	 * @param index The index number
	 * @return The name or QString::null
	 */
	QString getFactoryName(int index) const;
};

#endif
