/***************************************************************************
 *   Copyright (C) 2004 by Joris Guisson                                   *
 *   joris.guisson@student.luc.ac.be                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef RENAMEDIALOG_H
#define RENAMEDIALOG_H

#include <list>
#include <map>
#include "renamedialogbase.h"

class Song;
class RenameFileItem;
class MusicDatabase;


/**
 * @class RenameDialog
 * @brief Dialog for renaming
 * 
 * This dialog is for renaming a bunch of songs.
 */
class RenameDialog : public RenameDialogBase
{
	Q_OBJECT
	
	QString m_rename_scheme;
	std::map<Song*,RenameFileItem*> m_songs;
	MusicDatabase* m_db;
	
	typedef std::map<Song*,RenameFileItem*>::iterator RFItr;
public:
	RenameDialog(MusicDatabase* db,QWidget *parent = 0, const char *name = 0);
	virtual ~RenameDialog();
	
	/**
	 * Set the list of songs, this clears the old list.
	 * @param songs The list of songs
	 */
	void setSongs(const std::list<Song*> & songs);
	
	/**
	 * Clear all songs from the dialog.
	 */
	void clear();
	
	QString getRenameScheme() const {return m_rename_scheme;}
	void setRenameScheme(const QString & str);
	
public slots:
	virtual void onRenameStringChanged(const QString & s);
	virtual void onRenamePressed();
};

#endif
