/***************************************************************************
 *   Copyright (C) 2004 by Joris Guisson                                   *
 *   joris.guisson@pandora.be                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <klocale.h>
#include <kinputdialog.h>
#include <kmessagebox.h>
#include <kfiledialog.h>
#include <ktoolbar.h>
#include <qdragobject.h>
#include "playlistview.h"

#include "globals.h"
#include "kmusicmanager.h"
#include "mainwindow.h"
#include "musicdb/song.h"
#include "playlist.h"
#include "playlistmanager.h"
#include "playlisteditor.h"
#include "playlisteditorwidget.h"
#include "playlistselectionbox.h"
#include "playlistview.h"
#include "view/musiclistview.h"
#include "view/childview.h"
#include "player/musicplayer.h"
#include "musicdb/song.h"
#include "musicdb/musicdatabase.h"
#include "util/error.h"

PlayListEditor::PlayListEditor(KMusicManager* mm,const char *name)
	: Tool("tool/playlisteditor",mm, name),
		m_pl_man(0),m_widget(0),m_psb(0)
{
	m_pl_man = new PlayListManager(mm->getMusicDB(),this);
	
}


PlayListEditor::~PlayListEditor()
{
	delete m_widget;
}

void PlayListEditor::load()
{
	if (m_widget)
		return;
	
	KMusicManager* mm = getMusicMan();
	m_widget = new PlayListEditorWidget(0,"playlisted");
	
	connect(
			m_pl_man,SIGNAL(playListAdded(PlayList* )),
			this,SLOT(onPlayListAdded(PlayList* )));
	
	connect(
			m_pl_man,SIGNAL(playListRemoved(PlayList* )),
			this,SLOT(onPlayListRemoved(PlayList* )));
	
	connect(
			m_widget,SIGNAL(newPlayListClicked()),
			this,SLOT(newPlayList()));
	
	connect(
			m_widget,SIGNAL(playListClicked(const QString& )),
			this,SLOT(onPlayListClicked(const QString& )));
	
	connect(
			m_widget,SIGNAL(playListDoubleClicked(const QString& )),
			this,SLOT(onPlayListDoubleClicked(const QString& )));
	
	connect(
			m_widget,SIGNAL(deletePlayList(PlayList* )),
			this,SLOT(deletePlayList(PlayList* )));
	
	connect(
			m_widget,SIGNAL(songsDropped(QDropEvent*, PlayList* )),
			this,SLOT(onSongsDropped(QDropEvent*, PlayList* )));
	
	connect(
			this,SIGNAL(changePlayerSource(const QString&, bool )),
			mm,SLOT(changePlayerSource(const QString&, bool )));
	

	mm->getMainWindow()->addToolWindow(
			m_widget,KDockWidget::DockLeft,
			mm->getMainWindow()->getMainDockWidget(),35);
	
	addToPlayToolBar(mm->getMainWindow()->toolBar("PlayToolBar"));
	
	try
	{
		loadPlayLists();
	}
	catch (Error & e)
	{
		OutputLog() << "Error : " << e.errorMessage() << endl;
	}
}

void PlayListEditor::unload()
{
/*	try
	{
		savePlayLists();
	}
	catch (Error & e)
	{
	OutputLog() << "Error : " << e.errorMessage() << endl;
	}
*/
	
	delete m_widget;
	m_widget = 0;
	delete m_psb;
	m_psb = 0;
}

void PlayListEditor::onChildViewAdded(ChildView* cv)
{
	connect(
			cv->getMusicListView(),SIGNAL(createPlayList(const std::list< Song * >& )),
			this,SLOT(createPlayList(const std::list< Song * >& )));
}

void PlayListEditor::onPlayListAdded(PlayList* pl)
{
	m_widget->addPlayList(pl);
	m_psb->addPlayList(pl);
	PlayListView* plv = new PlayListView(pl,getMusicMan()->getMusicDB(),0);
	getMusicMan()->getMainWindow()->addChildView(plv,"playlist:"+pl->getName());
}

void PlayListEditor::onPlayListRemoved(PlayList* pl)
{
	m_psb->removePlayList(pl);
	KMusicManager* mm = getMusicMan();

	mm->getMainWindow()->removeChildView("playlist:"+pl->getName());
}

void PlayListEditor::addToPlayToolBar(KToolBar* ptb)
{
	new QLabel(i18n("  Current Playlist "),ptb);
	m_psb = new PlayListSelectionBox(ptb);
	
	connect(
		m_psb,SIGNAL(playListSelected(const QString& )),
		this,SLOT(onSourceSelected(const QString& )));
	
	connect(
		m_psb,SIGNAL(collectionSelected()),
		this,SLOT(onCollectionSelected()));
}

void PlayListEditor::onSourceSelected(const QString & name)
{
	changePlayerSource("playlist:"+name,false);
}

void PlayListEditor::onCollectionSelected()
{
	changePlayerSource("collection",false);
}

void PlayListEditor::savePlayLists()
{
	m_pl_man->savePlayLists(getMusicMan()->getDataDir());
}

void PlayListEditor::loadPlayLists()
{
	m_pl_man->loadPlayLists(getMusicMan()->getDataDir());
}

void PlayListEditor::onPlayListClicked(const QString& name)
{
	getMusicMan()->getMainWindow()->activateChildView(name);
}

void PlayListEditor::onPlayListDoubleClicked(const QString& name)
{
	getMusicMan()->getMainWindow()->activateChildView(name);
	changePlayerSource(name,true);
	if (name == "collection")
		m_psb->setCurrent(QString::null);
	else
		m_psb->setCurrent(name.mid(9));
}
	
void PlayListEditor::newPlayList()
{
	QString str = KInputDialog::getText(i18n("Name"),i18n("Enter the name of the playlist"));
	if (str != QString::null)
	{
		if (m_pl_man->containsPlayList(str))
		{
			KMessageBox::error(
				getMusicMan()->getMainWindow(),
				i18n("There allready is a playlist named %1").arg(str),
				i18n("Error"));
			
			return;
		}
			
		PlayList* pl = m_pl_man->newPlayList(str);
		
		pl->setFilename(getMusicMan()->getDataDir() + pl->getName() + ".m3u");
		
		PlayListView* plv = new PlayListView(pl,getMusicMan()->getMusicDB(),
											 getMusicMan()->getMainWindow());
		getMusicMan()->getMainWindow()->addChildView(plv,"playlist:" + str);
	}
}


void PlayListEditor::createPlayList(const std::list<Song*> & sel)
{
	QString str = KInputDialog::getText(i18n("Name"),i18n("Enter the name of the playlist"));
	if (str != QString::null)
	{
		if (m_pl_man->containsPlayList(str))
		{
			KMessageBox::error(
					getMusicMan()->getMainWindow(),
					i18n("There allready is a playlist named %1").arg(str),
					i18n("Error"));
			
			return;
		}
			
		PlayList* pl = m_pl_man->newPlayList(str);
		
		pl->setFilename(getMusicMan()->getDataDir() + pl->getName() + ".m3u");
		
		for (std::list<Song*>::const_iterator i = sel.begin();i != sel.end();i++)
		{
			pl->appendSong(*i);
		}
		
		pl->autosave();
	}
}


void PlayListEditor::deletePlayList(PlayList* pl)
{
	m_pl_man->removePlayList(pl->getName());
}

void PlayListEditor::onSongsDropped(QDropEvent* e,PlayList* pl)
{
	MusicDatabase* db = getMusicMan()->getMusicDB();
	QString filelist;
	if (QTextDrag::decode(e,filelist)) 
	{
		QStringList sl = QStringList::split("\n",filelist,false);
		
		for (QStringList::iterator i = sl.begin();i != sl.end();i++)
		{
			QString filename = *i;
			Song* s = db->getSong(filename);
			if (s)
			{
				pl->appendSong(s);
			}
		}
	}

	pl->autosave();
}



#include "playlisteditor.moc"
