/***************************************************************************
 *   Copyright (C) 2004 by Joris Guisson                                   *
 *   joris.guisson@pandora.be                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef PLAYLISTEDITOR_H
#define PLAYLISTEDITOR_H

#include <list>
#include "tools/tool.h"
#include "playlisteditorwidget.h"

class PlayList;
class PlayListManager;
class Song;
class QDropEvent;
class PlayListSelectionBox;
class KToolBar;
class MusicDatabase;


/**
@author Joris Guisson
*/
class PlayListEditor : public Tool
{
	Q_OBJECT
	
	PlayListManager* m_pl_man;
	PlayListEditorWidget* m_widget;
	PlayListSelectionBox* m_psb;
public:
	PlayListEditor(KMusicManager* mm,const char *name = 0);
	virtual ~PlayListEditor();

	QWidget* widget() {return m_widget;}
	
	virtual void load();
	virtual void unload();
	virtual void onChildViewAdded(ChildView* cv);
	
	void savePlayLists();
	void loadPlayLists();
	
private:
	void addToPlayToolBar(KToolBar* ptb);
	
public slots:
	void newPlayList();
	void createPlayList(const std::list<Song*> & sel);
	void onPlayListClicked(const QString& name);
	void onPlayListDoubleClicked(const QString& name);
	void deletePlayList(PlayList* pl);
	void onSongsDropped(QDropEvent* e,PlayList* pl);
	void onPlayListAdded(PlayList* pl);
	void onPlayListRemoved(PlayList* pl);
	void onSourceSelected(const QString & name);
	void onCollectionSelected();
	
signals:
	void changePlayerSource(const QString & name,bool next_song);
};

#endif
