/***************************************************************************
 *   Copyright (C) 2004 by Joris Guisson                                   *
 *   joris.guisson@student.luc.ac.be                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <klocale.h>
#include <kiconloader.h>
#include <kglobal.h>
#include <kfiledialog.h>
#include <kmessagebox.h>
#include <qdragobject.h>

#include "pledmenu.h"
#include "playlist.h"
#include "playlisteditor.h"
#include "playlisteditoritem.h"
#include "playlisteditorwidget.h"

PlayListEditorWidget::PlayListEditorWidget(QWidget *parent, const char *name)
		: KListBox(parent, name),m_menu(0),m_curr(0)
{
	setCaption(i18n("Playlist Editor"));
	
	m_pl_pix = KGlobal::iconLoader()->loadIcon("sound",KIcon::Desktop);
	
	m_coll = new QListBoxPixmap(
		this,
		KGlobal::iconLoader()->loadIcon("folder",KIcon::Desktop),
		i18n("Music Collection"));
	
	m_new_pl = new QListBoxPixmap(
		this,
		KGlobal::iconLoader()->loadIcon("filenew",KIcon::Desktop),
		i18n("New Playlist"));
	
	connect(
		this,SIGNAL(contextMenuRequested(QListBoxItem*, const QPoint& )),
		this,SLOT(showContextMenu(QListBoxItem*, const QPoint& )));
	
	connect(
		this,SIGNAL(doubleClicked(QListBoxItem* )),
		this,SLOT(onDoubleClicked(QListBoxItem* )));
	
	connect(this,SIGNAL(clicked(QListBoxItem* )),this,SLOT(onClicked(QListBoxItem* )));
	
	setAcceptDrops(true);
	setIcon(KGlobal::iconLoader()->loadIcon("sound",KIcon::Small));
}


PlayListEditorWidget::~PlayListEditorWidget()
{}

void PlayListEditorWidget::onClicked(QListBoxItem* item)
{
	if (!item)
		return;
	
	if (item == m_new_pl)
	{
		//newPlayList();
		newPlayListClicked();
	}
	else if (item == m_coll)
	{
		playListClicked("collection");
	}
	else
	{
		PlayListEditorItem* i = (PlayListEditorItem*)item;
		playListClicked("playlist:" + i->getPlayList()->getName());
	}
}

void PlayListEditorWidget::onDoubleClicked(QListBoxItem* item)
{
	if (item == m_new_pl)
	{
		newPlayListClicked();
	}
	else if (item == m_coll)
	{
		playListDoubleClicked("collection");
	}
	else
	{
		PlayListEditorItem* i = (PlayListEditorItem*)item;
		playListDoubleClicked("playlist:" + i->getPlayList()->getName());
	}
}

void PlayListEditorWidget::dropEvent(QDropEvent* e)
{
	QPoint pos = e->pos();
	PlayListEditorItem* item = dynamic_cast<PlayListEditorItem*>(itemAt(pos));
	if (item)
	{
		songsDropped(e,item->getPlayList());
	}
}

void PlayListEditorWidget::dragEnterEvent(QDragEnterEvent* e)
{
	e->accept(QTextDrag::canDecode(e));
}

void PlayListEditorWidget::addPlayList(PlayList* pl)
{
	new PlayListEditorItem(this,pl,m_pl_pix);
}

void PlayListEditorWidget::showContextMenu(QListBoxItem* item,const QPoint& pos)
{
	
	
	m_curr = item;
	if (!m_menu)
		m_menu = new PLEdMenu(this);
	
	if (item == m_new_pl)
	{
		m_menu->setPlayEnabled(false);
		m_menu->setDeleteEnabled(false);
		m_menu->setExportEnabled(false);
	}
	else if (item == m_coll)
	{
		m_menu->setPlayEnabled(true);
		m_menu->setDeleteEnabled(false);
		m_menu->setExportEnabled(false);
	}
	else
	{
		m_menu->setPlayEnabled(true);
		m_menu->setDeleteEnabled(true);
		m_menu->setExportEnabled(true);
	}
	
	m_menu->popup(pos);
}

void PlayListEditorWidget::playCurrent()
{
	if (!m_curr)
		return;
	
	if (m_curr == m_coll)
	{
		playListDoubleClicked("collection");
	}
	else
	{
		QString name = ((PlayListEditorItem*)m_curr)->getPlayList()->getName();
		playListDoubleClicked("playlist:" + name);
	}
}

void PlayListEditorWidget::deleteCurrent()
{
	if (!m_curr)
		return;
	
	deletePlayList(((PlayListEditorItem*)m_curr)->getPlayList());
	delete m_curr;
	m_curr = 0;
}

void PlayListEditorWidget::exportCurrent()
{
	if (!m_curr)
		return;
	
	QString file = KFileDialog::getOpenFileName(
		QString::null,
		"*.m3u|" + i18n("m3u Playlists(*.m3u)") + "\n*|" + i18n("All files"),
		this,
		i18n("Enter a filename"));
	
	if (file != QString::null)
	{
		if (!file.endsWith(".m3u"))
			file += ".m3u";
		
		PlayList* pl = ((PlayListEditorItem*)m_curr)->getPlayList();
		if (!pl->save(file))
		{
			KMessageBox::error(
				this,
				i18n("Can not save playlist to file %1").arg(file),
				i18n("Error"));
		}
	}
}


#include "playlisteditorwidget.moc"
