/***************************************************************************
 *   Copyright (C) 2004 by Joris Guisson                                   *
 *   joris.guisson@student.luc.ac.be                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef PLAYLISTEDITORWIDGET_H
#define PLAYLISTEDITORWIDGET_H

#include <klistbox.h>

class PLEdMenu;
class PlayList;
class PlayListEditorItem;

/**
@author Joris Guisson
*/
class PlayListEditorWidget : public KListBox
{
	Q_OBJECT
			
	QListBoxItem* m_new_pl;
	QListBoxItem* m_coll;
	QPixmap m_pl_pix;
	PLEdMenu* m_menu;
	QListBoxItem* m_curr;
	
	void dropEvent(QDropEvent* e);
	void dragEnterEvent(QDragEnterEvent* );
public:
	PlayListEditorWidget(QWidget *parent = 0, const char *name = 0);
	virtual ~PlayListEditorWidget();
	
public slots:
	void addPlayList(PlayList* pl);
	void onClicked(QListBoxItem* item);
	void onDoubleClicked(QListBoxItem* item);
	void showContextMenu(QListBoxItem* item,const QPoint& pos);
	void playCurrent();
	void deleteCurrent();
	void exportCurrent();

signals:
	/**
		Signal emitted when the user double clicks on a playlist.
		@param name The name of the playlist prepended with playlist: or collection
			if the user double clicks on the collection.
	*/
	void playListDoubleClicked(const QString & name);
	
	/**
		Signal emitted when the user clicks on a playlist.
		@param name The name of the playlist prepended with playlist: or collection
			if the user double clicks on the collection.
	*/
	void playListClicked(const QString & name);
	
	/**
		Signal emitted when the user clicks on new playlist.
	*/
	void newPlayListClicked();
	
	/**
		Signal emitted when the user requests to delete
		a playlist.
	*/
	void deletePlayList(PlayList* pl);
	
	/**
		Signal emitted when the user drops some songs on 
		a playlist.
	*/
	void songsDropped(QDropEvent* e,PlayList* pl);
};

#endif
