/***************************************************************************
 *   Copyright (C) 2004 by Joris Guisson                                   *
 *   joris.guisson@pandora.be                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <qdir.h>
#include <qstringlist.h>
#include <qfile.h>
#include <qfileinfo.h>
#include <kio/job.h>
#include "playlistmanager.h"
#include "globals.h"
#include "musicdb/musicdatabase.h"
#include "musicdb/song.h"



PlayListManager::PlayListManager(MusicDatabase* mdb,QObject* parent,const char *name)
		: QObject(parent, name),mdb(mdb)
{
}


PlayListManager::~PlayListManager()
{
	for (PlayListItr i = playlists.begin();i != playlists.end();i++)
	{
		delete i->second;
	}
}

PlayList* PlayListManager::newPlayList(const QString & name)
{
	if (getPlayList(name))
		return 0;
	
	PlayList* pl = new PlayList(name,mdb);
	playlists[name] = pl;
	playListAdded(pl);
	return pl;
}

PlayList* PlayListManager::loadPlayList(const QString & file)
{
	QFileInfo finfo(file);
	QString name = finfo.baseName();
	
	if (getPlayList(name))
		return 0;
	
	PlayList* pl = new PlayList(name,mdb);
	if (!pl->load(file,mdb))
	{
		delete pl;
		return 0;
	}
	else
	{
		playlists[name] = pl;
		playListAdded(pl);
		return pl;
	}
}

PlayList* PlayListManager::getPlayList(const QString & name)
{
	PlayListItr i = playlists.find(name);
	if (i == playlists.end())
		return 0;
	else
		return i->second;
}

bool PlayListManager::containsPlayList(const QString & name)
{
	return playlists.find(name) != playlists.end();
}
	
void PlayListManager::removePlayList(const QString & name)
{
	PlayListItr i = playlists.find(name);
	if (i != playlists.end())
	{
		PlayList* pl = i->second;
		if (pl->getFilename() != QString::null)
		{
			KIO::Job* job = KIO::del(pl->getFilename());
			connect(job,SIGNAL(result(KIO::Job* )),this,SLOT(delJobDone(KIO::Job* )));
		}
		playListRemoved(pl);
		delete pl;
		playlists.erase(i);
	}
}

void PlayListManager::delJobDone(KIO::Job* job)
{
	if (job->error() != 0)
		job->showErrorDialog(0);
}

void PlayListManager::savePlayLists(const QString & dir)
{
	for (PlayListItr i = playlists.begin();i != playlists.end();i++)
	{
		PlayList* pl = i->second;
		QString fn = dir + pl->getName() + ".m3u";
		pl->save(fn);
	}
}

void PlayListManager::loadPlayLists(const QString & dir)
{
	OutputLog() << "Loading playlists from dir " << dir << endl;
	QDir d(dir);
	QStringList li = d.entryList("*.m3u");
	for (QStringList::iterator i = li.begin();i != li.end();i++)
	{
		QString file = *i;
		OutputLog() << "Loading file " << file << endl;
		loadPlayList(dir + file);
	}
}


#include "playlistmanager.moc"
