/***************************************************************************
 *   Copyright (C) 2004 by Joris Guisson                                   *
 *   joris.guisson@pandora.be                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef PLAYLISTMANAGER_H
#define PLAYLISTMANAGER_H

#include <qobject.h>
#include <map>
#include "playlist.h"

class MusicDatabase;

namespace KIO
{
	class Job;
}

/**
@author Joris Guisson
*/
class PlayListManager : public QObject
{
	Q_OBJECT
	
	MusicDatabase* mdb;
	std::map<QString,PlayList*> playlists;
	
	typedef std::map<QString,PlayList*>::iterator PlayListItr;
public:
	PlayListManager(MusicDatabase* mdb,QObject* parent = 0,const char *name = 0);
	virtual ~PlayListManager();

	PlayList* newPlayList(const QString & name);
	PlayList* loadPlayList(const QString & file);
	PlayList* getPlayList(const QString & name);
	bool containsPlayList(const QString & name);
	void removePlayList(const QString & name);
	
	void savePlayLists(const QString & dir);
	void loadPlayLists(const QString & dir);
	
public slots:
	void delJobDone(KIO::Job* job);
	
signals:
	void playListAdded(PlayList* pl);
	void playListRemoved(PlayList* pl);
};

#endif
