/***************************************************************************
 *   Copyright (C) 2004 by Joris Guisson                                   *
 *   joris.guisson@pandora.be                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <klocale.h>
#include <kiconloader.h>
#include <kglobal.h>
#include <qpopupmenu.h>
#include "treeview.h"
#include "treeviewmenu.h"

#include "musicdb/musicdatabase.h"
#include "musicdb/dbdir.h"

#include "util/iojobscheduler.h"
#include "util/iodeletejob.h"

#include "view/songdragobject.h"

static void FillFileList(KFileTreeViewItem* item,std::list<Song*> & songs,MusicDatabase* db);

TreeView::TreeView(MusicDatabase* db,QWidget *parent, const char *name)
	: KFileTreeView(parent, name),m_db(db),m_menu(0)
{
	setCaption(i18n("Database Directories"));
	addColumn(i18n("Name"));
	setIcon(KGlobal::iconLoader()->loadIcon("folder",KIcon::Small));
	setFullWidth(true);
	setRootIsDecorated(true);
	setDragEnabled(true);
	
	for (DBDirItr i = m_db->beginDirs();i != m_db->endDirs();i++)
	{
		dirAdded(i.data());
	}
	connect(m_db,SIGNAL(dirAdded(DBDir* )),this,SLOT(dirAdded(DBDir* )));
	connect(m_db,SIGNAL(dirRemoved(DBDir* )),this,SLOT(dirRemoved(DBDir* )));
	
	connect(this,SIGNAL(contextMenuRequested(QListViewItem*, const QPoint&, int )),
			this,SLOT(showMenu(QListViewItem*, const QPoint&, int )));
	
	connect(this,SIGNAL(doubleClicked(QListViewItem* )),
			this,SLOT(onDoubleClicked(QListViewItem* )));
	
	connect(this,SIGNAL(clicked(QListViewItem* )),
			this,SLOT(onClicked(QListViewItem* )));
	
//	m_menu = new TreeViewMenu(this);
	setDragEnabled(true);
}


TreeView::~TreeView()
{}

void TreeView::dirAdded(DBDir* dir)
{
	KFileTreeBranch * b = addBranch(KURL(dir->getFullPath()),dir->getFullPath());
	b->setNameFilter("*.mp3 *.MP3 *.ogg *.OGG *.flac *.FLAC");
	b->setChildRecurse(true);
}

void TreeView::dirRemoved(DBDir* dir)
{
	removeBranch(branch(dir->getFullPath()));
}

void TreeView::deleteOnDisk()
{
	KFileTreeViewItem* curr = this->currentKFileTreeViewItem();
	if (curr)
	{
		IOJobScheduler::instance().addJob(new IODeleteJob(curr->path()));
	}
}

void TreeView::onDoubleClicked(QListViewItem* )
{
	KFileTreeViewItem* curr = this->currentKFileTreeViewItem();
	if (curr && curr->fileItem()->isFile())
	{
		playNow(m_db->getSong(curr->path()));
	}
}

void TreeView::onClicked(QListViewItem* )
{
	KFileTreeViewItem* curr = this->currentKFileTreeViewItem();
	if (curr && curr->fileItem()->isFile())
	{
		setCurrentSong(m_db->getSong(curr->path()));
	}
}

QDragObject* TreeView::dragObject()
{
	KFileTreeViewItem* curr = this->currentKFileTreeViewItem();
	if (!curr)
		return 0;
	
	SongDragObject* sdo = new SongDragObject(this);
	
	if (curr->isDir())
	{
		std::list<Song*> pl;
		FillFileList(curr,pl,m_db);
		
		for (std::list<Song*>::iterator i = pl.begin();i != pl.end();i++)
			sdo->addSong(*i);
	}
	else
	{
		sdo->addSong(m_db->getSong(curr->path()));
	}
	sdo->finalize();

	return sdo;
}

void TreeView::addToPlayQueue()
{
	KFileTreeViewItem* curr = this->currentKFileTreeViewItem();
	if (!curr)
		return;
	
	if (curr->isDir())
	{
		std::list<Song*> pl;
		FillFileList(curr,pl,m_db);
		
		for (std::list<Song*>::iterator i = pl.begin();i != pl.end();i++)
			playNext(*i);
	}
	else
	{
		playNext(m_db->getSong(curr->path()));
	}
}

void TreeView::createPlaylist()
{
	KFileTreeViewItem* curr = this->currentKFileTreeViewItem();
	if (!curr)
		return;
	
	if (!curr->isDir())
	{
		Song* s = m_db->getSong(curr->path());
		std::list<Song*> pl;
		pl.push_back(s);
		createPlaylist(pl);
	}
	else
	{
		std::list<Song*> pl;
		FillFileList(curr,pl,m_db);
		createPlaylist(pl);
	}
}

void TreeView::renameSong()
{
	KFileTreeViewItem* curr = this->currentKFileTreeViewItem();
	if (!curr || curr->isDir())
		return;
	
	renameSong(m_db->getSong(curr->path()));
}

void TreeView::showMenu(QListViewItem* ,const QPoint& pos, int )
{
	KFileTreeViewItem* curr = this->currentKFileTreeViewItem();
	if (!curr)
		return;

	if (!m_menu)
		m_menu = new TreeViewMenu(this);
	
	m_menu->popupMenu(pos,curr->isDir());
}

static void FillFileList(KFileTreeViewItem* item,std::list<Song*> & songs,MusicDatabase* db)
{
	QListViewItem* li = item->firstChild();

	while (li)
	{
		KFileTreeViewItem* vi = dynamic_cast<KFileTreeViewItem*>(li);
		if (vi->isDir())
		{
			FillFileList(vi,songs,db);
		}
		else
		{
			Song* s = db->getSong(vi->path());
			if (s)
				songs.push_back(s);
		}
		li = li->nextSibling();
	}
}

#include "treeview.moc"
