/***************************************************************************
 *   Copyright (C) 2004 by Joris Guisson                                   *
 *   joris.guisson@pandora.be                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef CHILDVIEW_H
#define CHILDVIEW_H

#include <kmdichildview.h>

class MusicListView;
class MusicListViewMenu;
class SearchBar;
class QListViewItem;
class MdiChildView;

/**
@author Joris Guisson
@brief View which shows a MusicListView with a SearchBar

This class shows a MusicListView together with a SearchBar. It also has MusicListViewMenu
which will be shown whenever the user request a context menu. The creation of the context menu
can be overriden in subclasses to provide an expanded menu.
*/
class ChildView : public QWidget
{
	Q_OBJECT
	
protected:
	QString m_caption;
	MusicListView* m_view;
	SearchBar* m_sb;
	MusicListViewMenu* m_menu;
	
	bool m_in_mdi;
	QString m_view_name;
	
	MdiChildView* m_mdi_win;
	QPixmap m_mdi_icon;
	
	/**
		Creates a context menu, can be overriden to 
		suply a different context menu.
	*/
	virtual MusicListViewMenu* createContextMenu();
public:
	ChildView(const QString &caption,QWidget *parent = 0, const char *name = 0);
	virtual ~ChildView();

	MusicListView* getMusicListView() {return m_view;}
	QString getMdiCaption() const {return m_caption;}
	const QPixmap & getMdiIcon() const {return m_mdi_icon;}
	void setMdiIcon(const QPixmap & pm) {m_mdi_icon = pm;}
	
	void setInMdi(bool in) {m_in_mdi = in;}
	bool isInMdi() const {return m_in_mdi;}
	
	void setViewName(const QString & vn) {m_view_name = vn;}
	QString getViewName() const {return m_view_name;}
	
	void setMdiWin(MdiChildView* v) {m_mdi_win = v;}
	MdiChildView* getMdiWin() {return m_mdi_win;}
	
public slots:
	void showContextMenu(QListViewItem* item,const QPoint& pos, int col);
	void updateView();
};

/**
	This is an ugly hack because KMdiMainFrm::removeWindowFromMdi isn't
	implemented.
*/
class MdiChildView : public KMdiChildView
{
	ChildView* m_cv;
public:
	MdiChildView(ChildView* cv,QWidget *parent = 0, const char *name = 0);
	virtual ~MdiChildView();
	
	ChildView* getChildView() {return m_cv;}
};

#endif
