/***************************************************************************
 *   Copyright (C) 2004 by Joris Guisson                                   *
 *   joris.guisson@pandora.be                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef MUSICLISTVIEW_H
#define MUSICLISTVIEW_H

#include <map>
#include <list>
#include <klistview.h>
#include "player/songprovider.h"
#include "musiclistviewconf.h"

class Song;
class MusicListViewItem;
class KConfig;

/**
@author Joris Guisson
*/
class MusicListView : public KListView,public SongProvider
{
	Q_OBJECT
			
	std::map<Song*,MusicListViewItem*> m_songs;
	QString m_filter;
	static MusicListViewConf m_vconf;
	int m_num_shown;
	QListViewItemIterator m_itr;
	
	QDragObject* dragObject();
	
	bool acceptDrag(QDropEvent* e) const;
public:
	MusicListView(QWidget *parent = 0, const char *name = 0);
	virtual ~MusicListView();
	
	static MusicListViewConf & getViewConfig() {return m_vconf;}
	static void setViewConfig(const MusicListViewConf & vc);
	
	unsigned int getNumSongs() const {return m_songs.size();}
	unsigned int getNumShown() const {return m_num_shown;}
	
	void saveSettings(KConfig* cfg);
	void loadSettings(KConfig* cfg);
	
	Song* firstSong();
	Song* nextSong(bool random,bool repeat);
	
	friend class MusicListViewMenu;
	

public slots:
	void setSearchFilter(const QString & str);
	void updateView();
	void clearView();
	void addSong(Song* s);
	void removeSong(Song* s);
	void refreshSelection();
	void onSongDoubleClick(QListViewItem* i);
	void onSelectionChanged(QListViewItem* item);
	void addSelectionToPlayQueue();
	void createPlayListFromSelection();
	void renameSelection();
	void deleteSelection();
	void onDropped(QDropEvent* e, QListViewItem*);

signals:
	void filtered(int num,int total);
	void songSelected(Song* s);
	void playNow(Song* s);
	void playNext(Song* s);
	void createPlayList(const std::list<Song*> & sel);
	void rename(const std::list<Song*> & sel);
	void songsDropped(const QStringList & songs);
};

#endif
