/***************************************************************************
 *   Copyright (C) 2004 by Joris Guisson                                   *
 *   joris.guisson@pandora.be                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <kkeydialog.h>
#include <kstatusbar.h>
#include <kconfig.h>

#include "mainwindow.h"

#include "kmusicmanager.h"
#include "actioncollection.h"
#include "trayicon.h"
#include "statusbar.h"

#include "view/childview.h"
#include "view/musiclistview.h"

#include "player/musicplayer.h"
#include "tools/toolmanager.h"

#include "player/timeslider.h"
#include "player/volumewidget.h"

#include "dialogs/musicmanprefdlg.h"


MainWindow::MainWindow(KMusicManager* mm)
	: KMdiMainFrm(0,"mainframe",KMdi::IDEAlMode),m_music_manager(mm)
{
	m_tray_icon = new TrayIcon(this);
	m_act_col = new ActionCollection(this);
	
	m_act_col->createActions(m_music_manager,this);
	m_act_col->buildSysTrayMenu(m_tray_icon->contextMenu());
	
	m_sb = new MMStatusBar(mm,this);
	createGUI(0);
	createPlayToolBar();
	connect(this,SIGNAL(viewActivated(KMdiChildView* )),
			this,SLOT(onViewActivated(KMdiChildView* )));
}


MainWindow::~MainWindow()
{}

void MainWindow::onViewActivated(KMdiChildView* cv)
{
	MdiChildView* c = dynamic_cast<MdiChildView*>(cv);
	if (c)
	{
		int num = c->getChildView()->getMusicListView()->getNumShown();
		int tot = c->getChildView()->getMusicListView()->getNumSongs();
		onFiltered(num,tot);
	}
}

void MainWindow::addChildView(ChildView* cv,const QString & name)
{
	CVItr i = m_views.find(name);
	if (i != m_views.end())
		return;
	
	m_views[name] = cv;
	cv->setViewName(name);
	
	MusicPlayer* player = m_music_manager->getPlayer();
	
	connect(cv->getMusicListView(),SIGNAL(playNow(Song* )),
			player,SLOT(playSong(Song* )));
	
	connect(cv->getMusicListView(),SIGNAL(playNext(Song* )),
			player,SLOT(enqueueSong(Song* )));
	
	connect(cv->getMusicListView(),SIGNAL(filtered(int, int )),
			this,SLOT(onFiltered(int, int )));
	
	m_music_manager->getToolManager()->onChildViewAdded(cv);
}

void MainWindow::removeChildView(const QString & name)
{
	CVItr i = m_views.find(name);
	if (i != m_views.end())
	{
		ChildView* cv = i->second;
		if (cv->getMdiWin())
			closeWindow(cv->getMdiWin());
		
		delete cv;
		m_views.erase(i);
	}
}

ChildView* MainWindow::getChildView(const QString & name)
{
	CVItr i = m_views.find(name);
	if (i != m_views.end())
		return i->second;
	else
		return 0;
}

void MainWindow::activateChildView(const QString & name)
{
	ChildView* cv = getChildView(name);
	if (cv)
	{
		showChildView(name);
		activateView(cv->getMdiWin());
		int num = cv->getMusicListView()->getNumShown();
		int tot = cv->getMusicListView()->getNumSongs();
		onFiltered(num,tot);
	}
}


void MainWindow::hideChildView(const QString & name)
{
	ChildView* cv = getChildView(name);
	if (cv && cv->isInMdi())
	{
		closeWindow(cv->getMdiWin());
		cv->setInMdi(false);
	}
}

void MainWindow::showChildView(const QString & name)
{
	ChildView* cv = getChildView(name);
	if (cv && !cv->isInMdi())
	{
		addWindow(new MdiChildView(cv,this),KMdi::StandardAdd);
		cv->setInMdi(true);
		int num = cv->getMusicListView()->getNumShown();
		int tot = cv->getMusicListView()->getNumSongs();
		filtered(num,tot);
	}
}

void MainWindow::childWindowCloseRequest(KMdiChildView* wnd)
{
	MdiChildView* cv = dynamic_cast<MdiChildView*>(wnd);
	if (cv)
	{
		hideChildView(cv->getChildView()->getViewName());
	}
}

void MainWindow::updateChildViews()
{
	for (CVItr i = m_views.begin();i != m_views.end();i++)
	{
		ChildView* cv = i->second;
		cv->updateView();
	}
}

void MainWindow::createPlayToolBar()
{
	KToolBar* ptb = toolBar("PlayToolBar");
	
	ptb->insertSeparator();
	
	MusicPlayer* player = m_music_manager->getPlayer();
	
	TimeSlider* time_slider = new TimeSlider(ptb);
	connect(player,SIGNAL(elapsed(int)),time_slider,SLOT(elapsed(int)));
	
	connect(player,SIGNAL(totalSongTimeChanged(int)),
			time_slider,SLOT(totalTimeChanged(int )));
	
	connect(time_slider,SIGNAL(seek(int )),player,SLOT(seek(int )));
	
	new QLabel("  ",ptb);
	
	VolumeWidget* volume = new VolumeWidget(ptb);
	volume->setVolume(player->getVolume());
	connect(volume,SIGNAL(volumeChanged(float )),player,SLOT(setVolume(float )));
	connect(player,SIGNAL(volumeChanged(float )),volume,SLOT(setVolume(float )));
	
	//ptb->show();
}

void MainWindow::onFiltered(int num,int tot)
{
	filtered(num,tot);
}

void MainWindow::enableSysTrayIcon(bool yes)
{
	m_enable_systray_icon = yes;
	if (yes)
	{
		m_tray_icon->show();
	}
	else
	{
		m_tray_icon->hide();
	}
}

void MainWindow::setActionStatus(bool random,bool repeat)
{
	m_act_col->getRandomPlay()->setChecked(random);
	m_act_col->getRepeatPlayList()->setChecked(repeat);
}

void MainWindow::optionsShowToolbar()
{
	// this is all very cut and paste code for showing/hiding the
	// toolbar
	if (m_act_col->getToolBarAction()->isChecked())
	{
		//	toolBar()->show();
		toolBar("PlayToolBar")->show();
	}
	else
	{
		//	toolBar()->hide();
		toolBar("PlayToolBar")->hide();
	}
}

void MainWindow::optionsShowStatusbar()
{
	// this is all very cut and paste code for showing/hiding the
	// statusbar
	if (m_act_col->getStatusBarAction()->isChecked())
		statusBar()->show();
	else
		statusBar()->hide();
}

void MainWindow::optionsConfigureKeys()
{
	KKeyDialog::configure(actionCollection(),this);
}

void MainWindow::optionsConfigureToolbars()
{
	// use the standard toolbar editor
#if defined(KDE_MAKE_VERSION)
# if KDE_VERSION >= KDE_MAKE_VERSION(3,1,0)
	saveMainWindowSettings(KGlobal::config(), autoSaveGroup());
# else
	saveMainWindowSettings(KGlobal::config());
# endif
#else
	saveMainWindowSettings(KGlobal::config());
#endif
}

void MainWindow::newToolbarConfig()
{
	// this slot is called when user clicks "Ok" or "Apply" in the toolbar editor.
	// recreate our GUI, and re-apply the settings (e.g. "text under icons", etc.)
	createGUI(0);

#if defined(KDE_MAKE_VERSION)
# if KDE_VERSION >= KDE_MAKE_VERSION(3,1,0)
	applyMainWindowSettings(KGlobal::config(), autoSaveGroup());
# else
	applyMainWindowSettings(KGlobal::config());
# endif
#else
	applyMainWindowSettings(KGlobal::config());
#endif
}

void MainWindow::optionsPreferences()
{
	// popup some sort of preference dialog, here
	MusicManPrefDlg dlg(m_music_manager);
	dlg.exec();
}

bool MainWindow::queryClose()
{
	if (m_enable_systray_icon)
	{
		hide();
		return false;
	}
	else
	{
		return true;
	}
}

bool MainWindow::queryExit()
{
	m_music_manager->onExit();
	return true;
}

void MainWindow::saveCVSettings(KConfig* cfg)
{
	for (CVItr i = m_views.begin();i != m_views.end();i++)
	{
		ChildView* cv = i->second;
		cfg->setGroup(i->first);
		cv->getMusicListView()->saveSettings(cfg);
		cfg->writeEntry("cv_in_mdi",cv->isInMdi());
	}
}

void MainWindow::loadCVSettings(KConfig* cfg)
{
	for (CVItr i = m_views.begin();i != m_views.end();i++)
	{
		ChildView* cv = i->second;
		cfg->setGroup(i->first);
		cv->getMusicListView()->loadSettings(cfg);
		if (cfg->readBoolEntry("cv_in_mdi",false))
		{
			showChildView(cv->getViewName());
		}
	}
}


#include "mainwindow.moc"

